Using Promises
==============

Here's an example of using promises. We use a toy example that allows us to run promises through their paces. What we demonstrate:

* creating a new promise
* resolving that at some unknown time
* doing something when promises resolve, both for each individually and for all together
* using a deferred to be able to choose when to continue with code set up earlier
* passing promises and/or their results down the chain of computation

Credits
-------

I was looking for a good tutorial on promises. There's great documentation and lots of good presentations, but I was looking for a tutorial that actually demonstrated a real asynchronous operation. The blog post that really was clear to me was [this series](http://abdulapopoola.com/tag/javascript-promise/) by Abdulfattah Popoola, especially [this one](http://abdulapopoola.com/2014/11/29/3-ways-to-start-using-promises/). After I had read that last post, I set out to basically copy and paste the code to see for myself how it worked. Then looked for a simple example I could apply his patterns to, and came up with this package delivery example.


What I learned
--------------

I had grokked promises already, but now it finally hit home to me how to think of `.then()` in the wider picture. This really happens as part of your control flow. You don't return anything from promises, you can't say:

    var finalResult = aPromise().then(return finalResult)

and just wait for the promise to complete. That's what you can do with fully synchronous code. Somehow I was expecting this kind of paradigm, when once you really understand what `.then()` is, this doesn't make any sense. No, conceptually I think `.then()` is more like syntax for creating windows in time. Each .then is a window in which you can do synchronous and asynchronous stuff; the interface between chained .then's ensures those windows will happen after each other in the order specified, _even if_ stuff inside them takes an unknown amount of time (provided you return promises from a .then()). 

How this is applicable
----------------------

Programming this little demo I saw how this could be useful for applications I build. I've built two fairly complex websites/apps that use a lot of event listeners for controlling the flow. Somewhere in one of Domenic Donicola's talks he mentioned this as a faulty paradigm, and I recognized myself in it. Using events to decouple your code is a pretty useful thing, and works well, and made it possible for me to write those applications. What you have then is:

    myFunc1(data){
        //does something
        trigger('step1');
    }

    myFunc2(data){
        //does something
        trigger('step2);
    }

    myFinalFunc(data){
        //does the last thing
    }

    window.on(step1, data, myFunc2);
    window.on(step2, data, myFinalFunc);
    myFunc1(data);


Like I said, this worked great, to a point. But this is really a bit of a misuse of event listeners, because these events fire only once. Using promises would be 'cleaner' and would add just a bit more 'flow' syntax explicitly in your code structure. The 'pseudo-observer' pattern mentioned by Abdulfattah Popoola in his blog post is an example of such.
